@@ -24,6 +24,7 @@
 extern int io_error;
 extern int keep_partial;
 extern int log_got_error;
+extern char *partial_dir;
 
 /**
  * Close all open sockets and files, allowing a (somewhat) graceful
@@ -36,12 +37,11 @@ void close_all(void)
 	int max_fd;
 	int fd;
 	int ret;
-	struct stat st;
+	STRUCT_STAT st;
 
 	max_fd = sysconf(_SC_OPEN_MAX) - 1;
 	for (fd = max_fd; fd >= 0; fd--) {
-		ret = fstat(fd,&st);
-		if (fstat(fd,&st) == 0) {
+		if ((ret = do_fstat(fd, &st)) == 0) {
 			if (is_a_socket(fd))
 				ret = shutdown(fd, 2);
 			ret = close(fd);
@@ -99,7 +99,7 @@ void _exit_cleanup(int code, const char *file, int line)
 
 	if (verbose > 3) {
 		rprintf(FINFO,"_exit_cleanup(code=%d, file=%s, line=%d): entered\n",
-			code, file, line);
+			code, safe_fname(file), line);
 	}
 
 	if (cleanup_child_pid != -1) {
@@ -117,9 +117,12 @@ void _exit_cleanup(int code, const char *file, int line)
 		cleanup_fname = NULL;
 		if (cleanup_fd_r != -1)
 			close(cleanup_fd_r);
-		if (cleanup_fd_w != -1)
+		if (cleanup_fd_w != -1) {
+			flush_write_file(cleanup_fd_w);
 			close(cleanup_fd_w);
-		finish_transfer(cleanup_new_fname, fname, cleanup_file, 0);
+		}
+		finish_transfer(cleanup_new_fname, fname, cleanup_file, 0,
+				!partial_dir);
 	}
 	io_flush(FULL_FLUSH);
 	if (cleanup_fname)
@@ -133,10 +136,12 @@ void _exit_cleanup(int code, const char *file, int line)
 	}
 
 	if (code == 0) {
-		if ((io_error & ~IOERR_VANISHED) || log_got_error)
-			code = RERR_PARTIAL;
-		else if (io_error)
+		if (io_error & IOERR_DEL_LIMIT)
+			code = RERR_DEL_LIMIT;
+		if (io_error & IOERR_VANISHED)
 			code = RERR_VANISHED;
+		if (io_error & IOERR_GENERAL || log_got_error)
+			code = RERR_PARTIAL;
 	}
 
 	if (code)
@@ -144,7 +149,7 @@ void _exit_cleanup(int code, const char *file, int line)
 
 	if (verbose > 2) {
 		rprintf(FINFO,"_exit_cleanup(code=%d, file=%s, line=%d): about to call exit(%d)\n",
-			ocode, file, line, code);
+			ocode, safe_fname(file), line, code);
 	}
 
 	close_all();
