@@ -25,6 +25,7 @@
 
 extern int verbose;
 extern int dry_run;
+extern int do_xfers;
 extern int log_format_has_i;
 extern int log_format_has_o_or_i;
 extern int daemon_log_format_has_i;
@@ -58,8 +59,8 @@ extern int csum_length;
 extern int ignore_times;
 extern int size_only;
 extern OFF_T max_size;
-extern int io_timeout;
 extern int io_error;
+extern int allowed_lull;
 extern int sock_f_out;
 extern int ignore_timeout;
 extern int protocol_version;
@@ -88,8 +89,6 @@ extern int backup_suffix_len;
 extern struct file_list *the_file_list;
 extern struct filter_list_struct server_filter_list;
 
-int allowed_lull = 0;
-
 static int deletion_count = 0; /* used to implement --max-delete */
 
 
@@ -151,7 +150,7 @@ static int delete_item(char *fname, int mode, int flags)
 		deletion_count--;
 		return 0;
 	}
-	if (!zap_dir || (errno != ENOTEMPTY && errno != EEXIST)) {
+	if (!zap_dir) {
 		rsyserr(FERROR, errno, "delete_file: rmdir %s failed",
 			full_fname(fname));
 		return -1;
@@ -170,10 +169,7 @@ static int delete_item(char *fname, int mode, int flags)
 			continue;
 
 		f_name_to(fp, buf);
-		if (delete_item(buf, fp->mode, flags & ~DEL_TERSE) != 0) {
-			flist_free(dirlist);
-			return -1;
-		}
+		delete_item(buf, fp->mode, flags & ~DEL_TERSE);
 	}
 	flist_free(dirlist);
 
@@ -185,7 +181,7 @@ static int delete_item(char *fname, int mode, int flags)
 	if (do_rmdir(fname) == 0) {
 		if (!(flags & DEL_TERSE))
 			log_delete(fname, mode);
-	} else if (errno != ENOTEMPTY && errno != ENOENT) {
+	} else if (errno != ENOTEMPTY && errno != EEXIST && errno != ENOENT) {
 		rsyserr(FERROR, errno, "delete_file: rmdir %s failed",
 			full_fname(fname));
 		return -1;
@@ -248,21 +244,25 @@ static void delete_in_dir(struct file_list *flist, char *fbuf,
 	if (link_stat(fbuf, &st, keep_dirlinks) < 0)
 		return;
 
-	if (one_file_system && file->flags & FLAG_TOP_DIR)
-		filesystem_dev = st.st_dev;
+	if (one_file_system) {
+		if (file->flags & FLAG_TOP_DIR)
+			filesystem_dev = st.st_dev;
+		else if (filesystem_dev != st.st_dev)
+			return;
+	}
 
 	dirlist = get_dirlist(fbuf, dlen, 0);
 
 	/* If an item in dirlist is not found in flist, delete it
 	 * from the filesystem. */
 	for (i = dirlist->count; i--; ) {
-		if (!dirlist->files[i]->basename)
+		struct file_struct *fp = dirlist->files[i];
+		if (!fp->basename || fp->flags & FLAG_MOUNT_POINT)
 			continue;
-		if (flist_find(flist, dirlist->files[i]) < 0) {
-			int mode = dirlist->files[i]->mode;
-			f_name_to(dirlist->files[i], delbuf);
-			if (delete_item(delbuf, mode, DEL_FORCE_RECURSE) < 0)
-				break;
+		if (flist_find(flist, fp) < 0) {
+			int mode = fp->mode;
+			f_name_to(fp, delbuf);
+			delete_item(delbuf, mode, DEL_FORCE_RECURSE);
 		}
 	}
 
@@ -276,7 +276,8 @@ static void do_delete_pass(struct file_list *flist)
 	char fbuf[MAXPATHLEN];
 	int j;
 
-	if (dry_run > 1) /* destination doesn't exist yet */
+	if (dry_run > 1 /* destination doesn't exist yet */
+	 || list_only)
 		return;
 
 	for (j = 0; j < flist->count; j++) {
@@ -291,6 +292,8 @@ static void do_delete_pass(struct file_list *flist)
 
 		delete_in_dir(flist, fbuf, file);
 	}
+	if (do_progress && !am_server)
+		rprintf(FINFO, "                    \r");
 }
 
 static int unchanged_attrs(struct file_struct *file, STRUCT_STAT *st)
@@ -665,7 +668,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 	if (only_existing && statret == -1 && stat_errno == ENOENT) {
 		/* we only want to update existing files */
 		if (verbose > 1) {
-			rprintf(FINFO, "not creating new file \"%s\"\n",
+			rprintf(FINFO, "not creating new %s \"%s\"\n",
+				S_ISDIR(file->mode) ? "directory" : "file",
 				safe_fname(fname));
 		}
 		return;
@@ -717,18 +721,12 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		return;
 	}
 
-	if (max_size && file->length > max_size) {
-		if (verbose > 1) {
-			rprintf(FINFO, "%s is over max-size\n",
-				safe_fname(fname));
-		}
-		return;
-	}
-
 	if (preserve_links && S_ISLNK(file->mode)) {
 #ifdef SUPPORT_LINKS
 		if (safe_symlinks && unsafe_symlink(file->u.link, fname)) {
 			if (verbose) {
+				if (the_file_list->count == 1)
+					fname = f_name(file);
 				rprintf(FINFO,
 					"ignoring unsafe symlink %s -> \"%s\"\n",
 					full_fname(fname),
@@ -821,15 +819,29 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		return;
 	}
 
-	if (preserve_hard_links && hard_link_check(file, ndx, HL_CHECK_MASTER))
+	if (preserve_hard_links
+	    && hard_link_check(file, ndx, fname, statret, &st,
+			       itemizing, code, HL_CHECK_MASTER))
 		return;
 
 	if (!S_ISREG(file->mode)) {
+		if (the_file_list->count == 1)
+			fname = f_name(file);
 		rprintf(FINFO, "skipping non-regular file \"%s\"\n",
 			safe_fname(fname));
 		return;
 	}
 
+	if (max_size && file->length > max_size) {
+		if (verbose > 1) {
+			if (the_file_list->count == 1)
+				fname = f_name(file);
+			rprintf(FINFO, "%s is over max-size\n",
+				safe_fname(fname));
+		}
+		return;
+	}
+
 	if (opt_ignore_existing && statret == 0) {
 		if (verbose > 1)
 			rprintf(FINFO, "%s exists\n", safe_fname(fname));
@@ -902,12 +914,14 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 				if (hard_link_one(file, ndx, fname, -1, &st,
 						  fnamecmpbuf, 1,
 						  itemizing && verbose > 1,
-						  code) == 0)
+						  code) == 0) {
+					if (preserve_hard_links
+					    && file->link_u.links) {
+						hard_link_cluster(file, ndx,
+								  itemizing,
+								  code);
+					}
 					return;
-				if (verbose) {
-					rsyserr(FINFO, errno, "link %s => %s",
-						full_fname(fnamecmpbuf),
-						safe_fname(fname));
 				}
 				match_level = 2;
 			}
@@ -923,8 +937,25 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 					}
 					match_level = 0;
 					statret = -1;
-				} else
-					set_perms(fname, file, NULL, 0);
+				} else {
+					if (itemizing) {
+						itemize(file, ndx, 0, &st,
+							ITEM_LOCAL_CHANGE, 0,
+							NULL);
+					} else if (verbose && code) {
+						rprintf(code, "%s\n",
+							safe_fname(fname));
+					}
+					set_perms(fname, file, NULL,
+						  maybe_PERMS_REPORT);
+					if (preserve_hard_links
+					    && file->link_u.links) {
+						hard_link_cluster(file, ndx,
+								  itemizing,
+								  code);
+					}
+					return;
+				}
 			} else if (compare_dest || match_level == 1) {
 				fnamecmp = fnamecmpbuf;
 				fnamecmp_type = i;
@@ -960,7 +991,9 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 	}
 
 	if (statret != 0) {
-		if (preserve_hard_links && hard_link_check(file, ndx, HL_SKIP))
+		if (preserve_hard_links
+		    && hard_link_check(file, ndx, fname, statret, &st,
+				       itemizing, code, HL_SKIP))
 			return;
 		if (stat_errno == ENOENT)
 			goto notify_others;
@@ -1001,7 +1034,7 @@ prepare_to_open:
 		statret = 0;
 	}
 
-	if (dry_run || read_batch || whole_file)
+	if (!do_xfers || read_batch || whole_file)
 		goto notify_others;
 
 	if (fuzzy_basis) {
@@ -1018,7 +1051,9 @@ prepare_to_open:
 			full_fname(fnamecmp));
 	    pretend_missing:
 		/* pretend the file didn't exist */
-		if (preserve_hard_links && hard_link_check(file, ndx, HL_SKIP))
+		if (preserve_hard_links
+		    && hard_link_check(file, ndx, fname, statret, &st,
+				       itemizing, code, HL_SKIP))
 			return;
 		statret = real_ret = -1;
 		goto notify_others;
@@ -1073,7 +1108,7 @@ notify_others:
 			fuzzy_file ? fuzzy_file->basename : NULL);
 	}
 
-	if (dry_run) {
+	if (!do_xfers) {
 		if (preserve_hard_links && file->link_u.links)
 			hard_link_cluster(file, ndx, itemizing, code);
 		return;
@@ -1104,27 +1139,26 @@ notify_others:
 
 void generate_files(int f_out, struct file_list *flist, char *local_name)
 {
-	int i, lull_mod;
+	int i;
 	char fbuf[MAXPATHLEN];
 	int itemizing, maybe_PERMS_REPORT;
 	enum logcode code;
+	int lull_mod = allowed_lull * 5;
 	int need_retouch_dir_times = preserve_times && !omit_dir_times;
 	int need_retouch_dir_perms = 0;
 	int save_only_existing = only_existing;
 	int save_opt_ignore_existing = opt_ignore_existing;
 	int save_do_progress = do_progress;
-
-	allowed_lull = read_batch ? 0 : (io_timeout + 1) / 2;
-	lull_mod = allowed_lull * 5;
+	int save_make_backups = make_backups;
 
 	if (protocol_version >= 29) {
 		itemizing = 1;
 		maybe_PERMS_REPORT = log_format_has_i ? 0 : PERMS_REPORT;
 		code = daemon_log_format_has_i ? 0 : FLOG;
 	} else if (am_daemon) {
-		itemizing = daemon_log_format_has_i && !dry_run;
+		itemizing = daemon_log_format_has_i && do_xfers;
 		maybe_PERMS_REPORT = PERMS_REPORT;
-		code = itemizing || dry_run ? FCLIENT : FINFO;
+		code = itemizing || !do_xfers ? FCLIENT : FINFO;
 	} else if (!am_server) {
 		itemizing = log_format_has_i;
 		maybe_PERMS_REPORT = log_format_has_i ? 0 : PERMS_REPORT;
@@ -1153,8 +1187,11 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 			: "enabled");
 	}
 
-	if (protocol_version < 29)
-		ignore_timeout = 1;
+	/* Since we often fill up the outgoing socket and then just sit around
+	 * waiting for the other 2 processes to do their thing, we don't want
+	 * to exit on a timeout.  If the data stops flowing, the receiver will
+	 * notice that and let us know via the redo pipe (or its closing). */
+	ignore_timeout = 1;
 
 	for (i = 0; i < flist->count; i++) {
 		struct file_struct *file = flist->files[i];
@@ -1169,7 +1206,8 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 		/* We need to ensure that any dirs we create have writeable
 		 * permissions during the time we are putting files within
 		 * them.  This is then fixed after the transfer is done. */
-		if (!am_root && S_ISDIR(file->mode) && !(file->mode & S_IWUSR)) {
+		if (!am_root && S_ISDIR(file->mode) && !(file->mode & S_IWUSR)
+		    && !list_only) {
 			int mode = file->mode | S_IWUSR; /* user write */
 			char *fname = local_name ? local_name : fbuf;
 			if (do_chmod(fname, mode & CHMOD_BITS) < 0) {
@@ -1185,6 +1223,8 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 
 		if (allowed_lull && !(i % lull_mod))
 			maybe_send_keepalive();
+		else if (!(i % 200))
+			maybe_flush_socket();
 	}
 	recv_generator(NULL, NULL, 0, 0, 0, code, -1);
 	if (delete_during)
@@ -1197,10 +1237,6 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 	ignore_times = 1;
 	make_backups = 0; /* avoid a duplicate backup for inplace processing */
 
-	/* We expect to just sit around now, so don't exit on a timeout.
-	 * If we really get a timeout then the other process should exit. */
-	ignore_timeout = 1;
-
 	if (verbose > 2)
 		rprintf(FINFO,"generate_files phase=%d\n",phase);
 
@@ -1218,6 +1254,7 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 	phase++;
 	only_existing = save_only_existing;
 	opt_ignore_existing = save_opt_ignore_existing;
+	make_backups = save_make_backups;
 
 	if (verbose > 2)
 		rprintf(FINFO,"generate_files phase=%d\n",phase);
@@ -1258,8 +1295,10 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 				continue;
 			recv_generator(f_name(file), file, i, itemizing,
 				       maybe_PERMS_REPORT, code, -1);
-			if (allowed_lull && !(j++ % lull_mod))
+			if (allowed_lull && !(++j % lull_mod))
 				maybe_send_keepalive();
+			else if (!(j % 200))
+				maybe_flush_socket();
 		}
 	}
 	recv_generator(NULL, NULL, 0, 0, 0, code, -1);
